<?php

namespace App\Http\Controllers;

use Exception;
use Carbon\Carbon;
use App\Helpers\Helper;
use App\Helpers\Stripe;
use App\Models\UserCard;
use Stripe\StripeClient;
use Illuminate\Http\Request;
use Stripe\Exception\CardException;
use App\Http\Requests\UserCardRequest;

class UserCardController extends Controller
{
    protected $return_url;
    protected $stripe;
    public function __construct()
    {
        $this->return_url = url('/card/callback');
        $this->stripe = new StripeClient(env('STRIPE_SECRET'));
    }
    /**
     * create
     *
     * @param  mixed $request
     * @return void
     */
    public function create(UserCardRequest $request)
    {

        try {
            $customer = auth()->user();
            if (!$customer->customer_stripe_id) {
                $customer->customer_stripe_id = $this->createStripeCustomer($customer);
            }
            $card = $this->stripe->customers->createSource($customer->customer_stripe_id, ['source' => $request->token]);
            $source = $this->stripe->setupIntents->create([
                'payment_method_types' => ['card'],
                'customer' => $customer->customer_stripe_id,
                'usage' => 'off_session',
                'confirm' => true,
                'payment_method' => $card->id,
                'return_url' => $this->return_url
            ]);
            // CARD WILL CREATE ON ACCEPTENCENCE OF INTENT
            $customer_card = $customer->card()->create([
                'card_id' => $source->status == 'succeeded' ? $card->id : $source->id,
                'last_4' => $card->last4,
                'brand' => $card->brand,
                'is_default' => $request->is_default,
                'status' => $source->status
            ]);
            if ($source->next_action) {
                switch ($source->next_action->type) {
                    case 'redirect_to_url':
                        $customer_card['url'] = $source->next_action->redirect_to_url->url;
                        break;
                    default:
                        # code...
                        break;
                }
            }
            return Helper::response($customer_card);
        } catch (Exception $e) {
            return Helper::response(null, 422, null, [$e->getMessage()]);
        }
    }
    public function showAll()
    {
        return Helper::response(["cards" => auth()->user()->card], 200);
    }
    public function delete(Request $request)
    {
        try {
            $customer = auth()->user();
            $this->stripe->customers->deleteSource(
                $customer->customer_stripe_id,
                $request->card_id,
                []
            );
            UserCard::where('card_id', $request->card_id)->delete();
            return Helper::response(null, 200, ['Card deleted successfully']);
        } catch (Exception $e) {
            return Helper::response(null, 422, [$e->getMessage()]);
        }
    }
    public function callback(Request $request)
    {
        $stripe = new Stripe();
        $intent = $stripe->getSetupIntent($request->setup_intent);
        $updateArr = [
            'status' => $intent->status,
            'response' => null,
            'deleted_at' => null
        ];
        if ($intent->status != 'succeeded' && $intent->last_setup_error) {
            $updateArr['response'] = @$intent->last_setup_error->message;
            $updateArr['deleted_at'] = Carbon::now();
        } else {
            $updateArr['card_id'] = $intent->payment_method;
        }
        UserCard::where('card_id', $intent->id)->update($updateArr);
        return view('confirm-payment', compact('intent'));
    }
    public function createStripeCustomer($customer)
    {
        $customer_stripe = $this->stripe->customers->create([
            'name' => $customer['name'],
            'email' => $customer['email'],
            'phone' => $customer['phone'],
            'payment_method' => 'pm_card_visa',
        ]);
        $customer->update([
            'customer_stripe_id' => $customer_stripe['id']
        ]);
        return $customer_stripe['id'];
    }
    public function createCardToken(Request $request)
    {
        try {
            $card = $this->stripe->tokens->create([
                'card' => [
                    'number' => $request->number,
                    'exp_month' => $request->exp_month,
                    'exp_year' => $request->exp_year,
                    'cvc' => $request->cvc,
                ],
            ]);

            // Card creation was successful
            return Helper::response(["card" => $card], 200, null, ['Card created successfully']);
        } catch (CardException $e) {
            // Handle Stripe card validation errors
            $errorMessage = $e->getMessage();
            $errorCode = $e->getCode();

            return Helper::response([], 400, false, ['Card creation failed. Error: ' . $errorMessage]);
        } catch (\Exception $e) {
            // Handle other exceptions
            return Helper::response([], 500, false, ['An error occurred while processing your request.']);
        }
    }
}
