<?php

namespace App\Http\Requests;

use App\Models\Order;
use App\Models\Product;
use App\Models\ProductAttribute;
use App\Models\PromoCode;
use Illuminate\Foundation\Http\FormRequest;
use SebastianBergmann\Type\NullType;

class   ProductRequest extends FormRequest
{
    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize()
    {
        return true;
    }

    public function rules()
    {
        $rules = [
            'product' => 'required|array|min:1',
            'product.*.id' => 'required|integer|exists:products,id',
            'product.*.quantity' => 'required|integer|min:1',
            'promo_code' => 'sometimes',
        ];

        // Optional color field validation
        if ($this->has('product.*.color')) {
            $rules['product.*.color'] = 'string'; // Add color rule
        }

        // Optional size field validation
        if ($this->has('product.*.size')) {
            $rules['product.*.size'] = 'string'; // Add size rule
        }

        return $rules;
    }

    public function withValidator($validator)
    {
        $validator->after(function ($validator) {
            $productData = $this->input('product');
            $promo_code = $this->input('promo_code');
            if ($promo_code) {
                // Check if the promo code exists and is valid
                $promo = PromoCode::where('code', $promo_code)
                    ->where('valid_to', '>', now()) // Check if the code is not expired
                    ->first();

                if (!$promo) {
                    $validator->errors()->add('promo_code', 'The selected promo code is invalid or expired.');
                }
                $promo_is_use = Order::where('promo_code', $promo_code)
                    ->where('user_id', auth()->user()->id) // Check if the code is not expired
                    ->first();

                if ($promo_is_use) {
                    $validator->errors()->add('promo_code', 'The selected promo code is already used.');
                }
            }
            foreach ($productData as $index => $item) {
                $productId = $item['id'];
                $quantity = $item['quantity'];
                $color = $item['color'] ?? null;
                $size = $item['size'] ?? null;

                // Check if the product exists in the database
                $product = Product::find($productId);

                if (!$product) {
                    $validator->errors()->add("product.$index.id", 'The selected product is invalid.');
                    continue; // Skip further validation for this product
                }

                // Check color if provided in the request
                if ($color !== null) {
                    $productAttribute = ProductAttribute::where('product_id', $productId)
                        ->where('color', $color)
                        ->first();

                    if (!$productAttribute) {
                        $validator->errors()->add("product.$index.color", "The selected color '{$color}' is not available for product '{$product->name}'.");
                    } elseif ($productAttribute->stock < $quantity) {
                        $validator->errors()->add("product.$index.color", "The selected color '{$color}' is out of stock for product '{$product->name}'.");
                    }
                }

                // Check size if provided in the request
                if ($size !== null) {
                    $productAttribute = ProductAttribute::where('product_id', $productId)
                        ->where('size', $size)
                        ->first();

                    if (!$productAttribute) {
                        $validator->errors()->add("product.$index.size", "The selected size '{$size}' is not available for product '{$product->name}'.");
                    } elseif ($productAttribute->stock < $quantity) {
                        $validator->errors()->add("product.$index.size", "The selected size '{$size}' is out of stock for product '{$product->name}'.");
                    }
                }
            }
        });
    }



    /**
     * Get custom error messages for validator errors.
     *
     * @return array
     */
    public function messages()
    {
        return [
            'product.required' => 'The product field is required.',
            'product.array' => 'The product must be an array.',
            'product.min' => 'The product must have at least one item.',
            'product.*.id.required' => 'The product ID is required.',
            'product.*.id.integer' => 'The product ID must be an integer.',
            'product.*.id.exists' => 'The selected product ID is invalid or does not exist in the products table.',
            'product.*.quantity.required' => 'The product quantity is required.',
            'product.*.quantity.integer' => 'The product quantity must be an integer.',
            'product.*.quantity.min' => 'The product quantity must be at least 1.',
        ];
    }
}
